==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/base/treepicdata.cpp <==
/*
 * Copyright (C) 2005-2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treepicdata.h"

#include <iostream>
#include <iomanip>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include "xmldefs.h"

#include "treegroupbase.h"
#include "treegrouproot.h"

#include "exif.h"

#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"
#include "basetools.h"

std::string TreePicData::getText()
{
  return(path);
}

std::string TreePicData::getSomeText(eText whichText)
{
  switch (whichText)
  {
   case TEXT_EXIF:
    {
      std::string mypath = getBasepath() + path;
      ExifInfo*   exif   = new ExifInfo(mypath);
      std::string tmp    = "";
      if (exif->hasEXIF())
      {
        tmp = "----------- EXIF -------------\n";
        tmp += exif->getInfo();
      }
      delete exif;

      return(tmp);
    }
   default:
    return("");
  }
}

TreePicData::TreePicData(TreeItem* prev, TreeGroupBase* parent) : TreePicBase(prev, parent)
{
  dt.nDate = 0;
  dt.nTime = 0;
  path     = "";
  keywords = "";
  comment  = "";
  fHidden  = false;
}

void TreePicData::setHidden()
{
  TreeItem* p = itemParent;
  
  while (p->itemParent != (TreeItem*)0)
    p = p->itemParent;  
  
  std::string                     kw    = strU(getKeywords());
  StringSet*                      words = ((TreeGroupRoot*)p)->getHiddenKeywords();
  std::set<std::string>::iterator i     = words->begin();
  
  while (i != words->end() && !fHidden)
  {            
    if (kw.find(*i) != std::string::npos)
      fHidden = true;
      
    i++;
  }     
}

TreePicData::~TreePicData()
{
}

int TreePicData::loadFromXML_all(SimpleXMLTransfer*    source,
                                 LinkInfo*             links)
{
  TreeItem::loadFromXML_all(source, links);
  
  dt.nTime = source->attributeAsInt(XMLNAME_TIME, 0);
  dt.nDate = source->attributeAsInt(XMLNAME_DATE, 0);
  path     = source->attribute(XMLNAME_PATH,     "");
  keywords = source->attribute(XMLNAME_KEYWORDS, "");
  comment  = source->attribute(XMLNAME_COMMENT,  "");
    
  return(0);
}
   
int TreePicData::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  // clean up keywords
  while (keywords.length() && keywords[0] == ',')
    keywords = keywords.substr(1);
  while (keywords.length() && keywords[keywords.length()-1] == ',')
    keywords.resize(keywords.length()-1);  
  
  TreeItem::saveToXML_all(dest, fConvLinks, nVersion);

  dest->setName(XMLNAME_SAVE_ITEM);
  dest->addAttribute(XMLNAME_SAVE_TYPE,     XMLNAME_SAVE_TYPE_PIC);
  dest->addAttribute(XMLNAME_SAVE_PATH,     path);
  if (!keywords.empty())
    dest->addAttribute(XMLNAME_SAVE_KEYWORDS, keywords);
  
  if (!comment.empty())
  {
    if (nVersion != 0)
      comment = ConvertComment(comment, nVersion);
    
    dest->addAttribute(XMLNAME_SAVE_COMMENT, comment);
  }
  
  if (dt.nTime != 0)
    dest->addAttribute(XMLNAME_SAVE_TIME,     dt.nTime);
  if (dt.nDate != 0)
    dest->addAttribute(XMLNAME_SAVE_DATE,     dt.nDate);

  return(0);
}
   
enum TreeItem::eItemLinkType TreePicData::getLinkType()
{
  return(DATA);
}
   
std::string TreePicData::getPath()
{
  return(path);
}

std::string TreePicData::getComment()
{
  return(comment);
}

std::string TreePicData::getKeywords()
{
  return(keywords);
}

std::string TreePicData::getTimeString()
{
  return(dt.getTimeAsString());
}

std::string TreePicData::getDateString()
{
  return(dt.getDateAsString());
}

bool TreePicData::hasTime()
{
  return(dt.isTimeSet());
}

bool TreePicData::hasDate()
{
  return(dt.isDateSet());
}

JWWDateTime TreePicData::getDateTime()
{
  return(dt);
}

std::string TreePicData::getDescr(int level, bool fShort)
{
  const unsigned int maxlen = 50;
  
  std::string tmp = "";
  bool        fDateTime = false;
  
  if (level >= 1 && (hasDate() || hasTime())) 
  {
    if (!fShort)
      tmp = "<p>";
  
    if (hasDate())
    {
      tmp += getDateString() + " ";
      fDateTime = true;
    }
    
    if (hasTime())
    {
      tmp += getTimeString();
      fDateTime = true;
    }

    if (!fShort)
      tmp += "</p>";
  }
  
  if (level >= 2)
  {
    if (fDateTime && comment.length() > 0)
    {
      tmp += "\n";
    }
    
    if (comment.length())
    {
      if (fShort)
      {
        if (comment.length() > maxlen)
          tmp += comment.substr(0, maxlen-3) + "...";
        else
          tmp += comment;
      }
      else
        tmp += getCommentHtml();              
    }
  }
  
  if (level >= 3)
  {
    std::vector<std::string> kwlist;
    unsigned int size;
    kw_list(kwlist);
    size = kwlist.size();
    if (size)
    {
      if (!fShort)
        tmp += "<p>";
      
      if (fShort && size > 10)
        size = 10;
      if (tmp.length() && kwlist.size() > 0)
        tmp += "\n";
      for (unsigned int n=0; n<size; n++)
        tmp += "\n" + kwlist[n];
      if (size != kwlist.size())
        tmp += "  ...";
      
      if (!fShort)
        tmp += "</p>";
    }
  }
  
  if (level >= 4)
  {
    std::string t = getText();
    if (t.length())
    {
      tmp += "\n";
      if (!fShort)
        tmp += "<p>";      
      tmp += t;
      if (!fShort)
        tmp += "</p>";      
    }
  }
  
  if (level >= 5)
  {
    std::string t = getSomeText(TreeItem::TEXT_EXIF);
    // replace newline with <br/>
    std::string tout = "";
    bool fNL = false;
    for (std::string::size_type n=0; n<t.length(); n++)
    {
      if (t[n] == '\n')
      {
        if (!fNL)
          tout += "<br/>";
        fNL = true;
      }
      else
      {
        fNL = false;
        tout += t[n];
      }
    }
    if (t.length())
    {
      tmp += "\n";
      if (fShort)
        tmp += t;
      else
        tmp += "<p>" + tout + "</p>";      
    }
  }
  
  return(tmp);
}

int  TreePicData::connectLinks(LinkInfo*  links)
{
  // --- count my own links -----------------------------------------
  return(TreeItem::countCommentLinks(links, true));
}

void TreePicData::setKeywords(const char* nK)
{
  keywords = collapse_space(trim(nK));
}

void TreePicData::setComment(const char* nC)
{
  comment = nC;
}

int TreePicData::setDate(const char* nD)
{
  return(dt.setDate(nD));
}

int TreePicData::setTime(const char* nT)
{
  return(dt.setTime(nT));
}

void TreePicData::changeBasepath(std::string oldbp, 
                                 std::string newbp)
{
  std::string tmp = oldbp + path;
  path = tmp.substr(newbp.length());
}

void TreePicData::setData(std::string       newpath,
                          std::string       basepath,
                          eDateTimeSource   eDTS)
{
  bool fSet = false;
  ExifInfo* exif = new ExifInfo(newpath);
  
  // set time and date
  if (eDTS != eFILE)
  {
    // try exif
    if (exif->hasEXIF())
    {
      std::string tmp = exif->getDateTime();
      if (tmp.length() == 19)
      {
        dt.setDate(tmp.substr(0, 10).c_str());
        dt.setTime(tmp.substr(11).c_str());
        fSet = true;
      }      
    }
  }
  
  if (!fSet && eDTS != eEXIF)
  {
    // take remaining data from file
    struct stat     Attribs;
    struct tm       timestruct;
    
    stat(newpath.c_str(), &Attribs);

#ifndef WIN32
    localtime_r(&Attribs.st_mtime, &timestruct);
#endif
            
    dt.nDate = JWWDateTime::assembleDate(1900 + timestruct.tm_year, timestruct.tm_mon+1, timestruct.tm_mday);
    
    if (eDTS != eEXIFFILEDATE)
      dt.nTime = JWWDateTime::assembleTime(timestruct.tm_hour, timestruct.tm_min, timestruct.tm_sec);
  }
  
  // read additional data (comment, keywords, ...)
  comment  = exif->ReadComment("");
  keywords = "";    
  {
    std::set<std::string> kwset;
    exif->ReadKeywords(kwset);
    kw_add(kwset, true);
  }
    
  delete exif;        
  path = newpath.substr(basepath.length());
}

JWWDateTime TreePicData::getDT()
{
  return(dt);
}

unsigned int TreePicData::countMissingFiles(std::string basepath)
{
  if (FileTools::fileExists(basepath + path))
    return(0);
  else
  {
    std::cerr << "File does not exist: " << basepath << path << "\n";
    return(1);
  }
}

unsigned int TreePicData::getYear()
{
  return(dt.getYear());
}

void TreePicData::timeshift(int nSec)
{
  dt.shiftSec(nSec);
}

std::string TreePicData::getBasepath()
{
  TreeGroupBase* p = itemParent;
  
  while (p->itemParent != (TreeGroupBase*)0)
  {
    p = p->itemParent;
  }
  
  if (p->getLinkType() != ROOTDATA)
  {
    return("[internal error]");
  }
  else
  {
    return(((TreeGroupRoot*)p)->getBasepath());
  }
}

bool TreePicData::isHidden()
{
  return(fHidden);
}

void TreePicData::ImportMetadata(const std::string basepath)
{
  ExifInfo* exif = new ExifInfo(basepath + path);

  // merge keywords
  {
    std::set<std::string> kwset;
    exif->ReadKeywords(kwset);
    kw_add(kwset, true);
  }
  
  // merge comments
  comment = exif->ReadComment(comment);
    
  delete exif;
}

==> fotoalbum-0.104.tar.bz2: fotoalbum-0.104/src/base/treepicdata.cpp <==
/*
 * Copyright (C) 2005-2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treepicdata.h"

#include <iostream>
#include <iomanip>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include "xmldefs.h"
#include "utf_conv.h"

#include "treegroupbase.h"
#include "treegrouproot.h"

#include "exif.h"

#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"
#include "../shared/utf8.h"
#include "basetools.h"

std::string TreePicData::getText()
{
  return(path);
}

std::string TreePicData::getSomeText(eText whichText)
{
  switch (whichText)
  {
   case TEXT_EXIF:
    {
      std::string mypath = getBasepath() + path;
      ExifInfo*   exif   = new ExifInfo(mypath);
      std::string tmp    = "";
      if (exif->hasEXIF())
      {
        tmp = "----------- EXIF -------------\n";
        tmp += exif->getInfo();
      }
      delete exif;

      return(tmp);
    }
   default:
    return("");
  }
}

TreePicData::TreePicData(TreeItem* prev, TreeGroupBase* parent) : TreePicBase(prev, parent)
{
  dt.nDate = 0;
  dt.nTime = 0;
  path     = "";
  keywords = "";
  comment  = "";
  fHidden  = false;
}

void TreePicData::setHidden()
{
  TreeItem* p = itemParent;
  
  while (p->itemParent != (TreeItem*)0)
    p = p->itemParent;  
  
  std::string                     kw    = getKeywords();
  StringSet*                      words = ((TreeGroupRoot*)p)->getHiddenKeywords();
  std::set<std::string>::iterator i     = words->begin();
  
  while (i != words->end() && !fHidden)
  {            
    if (kw.find(*i) != std::string::npos)
      fHidden = true;
      
    i++;
  }     
}

TreePicData::~TreePicData()
{
}

int TreePicData::loadFromXML_all(SimpleXMLTransfer*    source,
                                 LinkInfo*             links)
{
  TreeItem::loadFromXML_all(source, links);
  
  dt.nTime = source->attributeAsInt(XMLNAME_TIME, 0);
  dt.nDate = source->attributeAsInt(XMLNAME_DATE, 0);
  path     = source->attribute(XMLNAME_PATH,     "");
  DecodePath(path);
  keywords = source->attribute(XMLNAME_KEYWORDS, "");
  comment  = source->attribute(XMLNAME_COMMENT,  "");
    
  return(0);
}
   
int TreePicData::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  // clean up keywords
  while (keywords.length() && keywords[0] == ',')
    keywords = keywords.substr(1);
  while (keywords.length() && keywords[keywords.length()-1] == ',')
    keywords.resize(keywords.length()-1);  
  
  TreeItem::saveToXML_all(dest, fConvLinks, nVersion);

  dest->setName(XMLNAME_SAVE_ITEM);
  dest->addAttribute(XMLNAME_SAVE_TYPE,     XMLNAME_SAVE_TYPE_PIC);
  dest->addAttribute(XMLNAME_SAVE_PATH,     EncodePath(getPath()));
  if (!keywords.empty())
    dest->addAttribute(XMLNAME_SAVE_KEYWORDS, keywords);
  
  if (!comment.empty())
  {
    if (nVersion != 0)
      comment = ConvertComment(comment, nVersion);
    
    dest->addAttribute(XMLNAME_SAVE_COMMENT, comment);
  }
  
  if (dt.nTime != 0)
    dest->addAttribute(XMLNAME_SAVE_TIME,     dt.nTime);
  if (dt.nDate != 0)
    dest->addAttribute(XMLNAME_SAVE_DATE,     dt.nDate);

  return(0);
}
   
enum TreeItem::eItemLinkType TreePicData::getLinkType()
{
  return(DATA);
}
   
std::string TreePicData::getPath()
{
  return(path);
}

std::string TreePicData::getComment()
{
  return(comment);
}

std::string TreePicData::getKeywords()
{
  return(keywords);
}

std::string TreePicData::getTimeString()
{
  return(dt.getTimeAsString());
}

std::string TreePicData::getDateString()
{
  return(dt.getDateAsString());
}

bool TreePicData::hasTime()
{
  return(dt.isTimeSet());
}

bool TreePicData::hasDate()
{
  return(dt.isDateSet());
}

JWWDateTime TreePicData::getDateTime()
{
  return(dt);
}

std::string TreePicData::getDescr(int level, bool fShort)
{
  const unsigned int maxlen = 50;
  
  std::string tmp = "";
  bool        fDateTime = false;
  
  if (level >= 1 && (hasDate() || hasTime())) 
  {
    if (!fShort)
      tmp = "<p>";
  
    if (hasDate())
    {
      tmp += getDateString() + " ";
      fDateTime = true;
    }
    
    if (hasTime())
    {
      tmp += getTimeString();
      fDateTime = true;
    }

    if (!fShort)
      tmp += "</p>";
  }
  
  if (level >= 2)
  {
    if (fDateTime && comment.length() > 0)
    {
      tmp += "\n";
    }
    
    if (comment.length())
    {
      if (fShort)
      {
        if (comment.length() > maxlen)
          tmp += comment.substr(0, maxlen-3) + "...";
        else
          tmp += comment;
      }
      else
        tmp += getCommentHtml();              
    }
  }
  
  if (level >= 3)
  {
    std::vector<std::string> kwlist;
    unsigned int size;
    kw_list(kwlist);
    size = kwlist.size();
    if (size)
    {
      if (!fShort)
        tmp += "<p>";
      
      if (fShort && size > 10)
        size = 10;
      if (tmp.length() && kwlist.size() > 0)
        tmp += "\n";
      for (unsigned int n=0; n<size; n++)
        tmp += "\n" + kwlist[n];
      if (size != kwlist.size())
        tmp += "  ...";
      
      if (!fShort)
        tmp += "</p>";
    }
  }
  
  if (level >= 4)
  {
    std::string t = getText();
    if (t.length())
    {
      tmp += "\n";
      if (!fShort)
        tmp += "<p>";      
      tmp += t;
      if (!fShort)
        tmp += "</p>";      
    }
  }
  
  if (level >= 5)
  {
    std::string t = getSomeText(TreeItem::TEXT_EXIF);
    // replace newline with <br/>
    std::string tout = "";
    bool fNL = false;
    for (std::string::size_type n=0; n<t.length(); n++)
    {
      if (t[n] == '\n')
      {
        if (!fNL)
          tout += "<br/>";
        fNL = true;
      }
      else
      {
        fNL = false;
        tout += t[n];
      }
    }
    if (t.length())
    {
      tmp += "\n";
      if (fShort)
        tmp += t;
      else
        tmp += "<p>" + tout + "</p>";      
    }
  }
  
  return(tmp);
}

int  TreePicData::connectLinks(LinkInfo*  links)
{
  // --- count my own links -----------------------------------------
  return(TreeItem::countCommentLinks(links, true));
}

void TreePicData::setKeywords(const char* nK)
{
  keywords = UTF8::collapse_space(UTF8::trim(nK));
}

void TreePicData::setComment(const char* nC)
{
  comment = nC;
}

int TreePicData::setDate(const char* nD)
{
  return(dt.setDate(nD));
}

int TreePicData::setTime(const char* nT)
{
  return(dt.setTime(nT));
}

void TreePicData::changeBasepath(std::string oldbp, 
                                 std::string newbp)
{
  std::string tmp = oldbp + path;
  path = tmp.substr(newbp.length());
}

void TreePicData::setData(std::string       newpath,
                          std::string       basepath,
                          eDateTimeSource   eDTS)
{
  bool fSet = false;
  ExifInfo* exif = new ExifInfo(newpath);
  
  // set time and date
  if (eDTS != eFILE)
  {
    // try exif
    if (exif->hasEXIF())
    {
      std::string tmp = exif->getDateTime();
      if (tmp.length() == 19)
      {
        dt.setDate(tmp.substr(0, 10).c_str());
        dt.setTime(tmp.substr(11).c_str());
        fSet = true;
      }      
    }
  }
  
  if (!fSet && eDTS != eEXIF)
  {
    // take remaining data from file
    struct stat     Attribs;
    struct tm       timestruct;
    
    stat(newpath.c_str(), &Attribs);

#ifndef WIN32
    localtime_r(&Attribs.st_mtime, &timestruct);
#endif
            
    dt.nDate = JWWDateTime::assembleDate(1900 + timestruct.tm_year, timestruct.tm_mon+1, timestruct.tm_mday);
    
    if (eDTS != eEXIFFILEDATE)
      dt.nTime = JWWDateTime::assembleTime(timestruct.tm_hour, timestruct.tm_min, timestruct.tm_sec);
  }
  
  // read additional data (comment, keywords, ...)
  comment  = exif->ReadComment("");
  keywords = "";    
  {
    std::set<std::string> kwset;
    exif->ReadKeywords(kwset);
    kw_add(kwset, true);
  }
    
  delete exif;        
  path = newpath.substr(basepath.length());
}

JWWDateTime TreePicData::getDT()
{
  return(dt);
}

unsigned int TreePicData::countMissingFiles(std::string basepath)
{
  if (FileTools::fileExists(basepath + path))
    return(0);
  else
  {
    std::cerr << "File does not exist: " << basepath << path << "\n";
    return(1);
  }
}

unsigned int TreePicData::getYear()
{
  return(dt.getYear());
}

void TreePicData::timeshift(int nSec)
{
  dt.shiftSec(nSec);
}

std::string TreePicData::getBasepath()
{
  TreeGroupBase* p = itemParent;
  
  while (p->itemParent != (TreeGroupBase*)0)
  {
    p = p->itemParent;
  }
  
  if (p->getLinkType() != ROOTDATA)
  {
    return("[internal error]");
  }
  else
  {
    return(((TreeGroupRoot*)p)->getBasepath());
  }
}

bool TreePicData::isHidden()
{
  return(fHidden);
}

void TreePicData::ImportMetadata(const std::string basepath)
{
  ExifInfo* exif = new ExifInfo(basepath + path);

  // merge keywords
  {
    std::set<std::string> kwset;
    exif->ReadKeywords(kwset);
    kw_add(kwset, true);
  }
  
  // merge comments
  comment = exif->ReadComment(comment);
    
  delete exif;
}

void TreePicData::DecodePath(std::string& path)
{
  int length = path.length();
  int o = 0;
  char ch;
  for (int i=0; i<length; )
  {
    ch = path.at(i);
    if (ch != '#')
    {
      path[o++] = ch;
      i++;
    }
    else
    {
      path[o++] = (char)( (hex2int(path.at(i+1))<<4) + hex2int(path.at(i+2)) );
      i+= 3;
    }       
  }
  if (o < length)
    path.resize(o);
}

std::string TreePicData::EncodePath(std::string path)
{
  int length = path.length();
  char ch;
  std::string strBuf;
  
  for (int i = 0; i < length; i++)
  {
    ch = path.at(i);
    
    if (ch != ' ' && (ch < '(' || ch > '~' || ch == '<' || ch == '>'))
    {
      if (strBuf.empty())
        strBuf = path.substr(0, i);
      strBuf.push_back('#');
      strBuf.append(itoHexStr( (int)((unsigned char)(ch)) ));
    }
    else if (!strBuf.empty())
      strBuf.push_back(ch);    
  }
  
  if (strBuf.empty())
    return(path);
  else
    return(strBuf);
}
